﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FlightPlan.Optimizer.Airports
{
    /// <summary>
    /// Represents an Airport.
    /// </summary>
    public class Airport
    {
        #region - Constructors -

        /// <summary>
        /// Creates a new instance of Airport.
        /// </summary>
        public Airport()
        {
            this.Name     = "";
            this.Waypoint = null;
        }

        /// <summary>
        /// Creates a new instance of Airport.
        /// </summary>
        /// <param name="name">The airport name.</param>
        /// <param name="waypoint">The airport waypoint.</param>
        public Airport(String name, Waypoint waypoint)
        {
            this.Name     = name;
            this.Waypoint = waypoint;
        }

        /// <summary>
        /// Creates a new instance of Airport.
        /// </summary>
        /// <param name="name">The airport name.</param>
        /// <param name="lat">The airport latitude.</param>
        /// <param name="lon">The airport longitude</param>
        public Airport(String name, double lat, double lon)
            :this(name, new Waypoint(null, null, new GPSCoords(lat, true), new GPSCoords(lon, false), false, null))
        {
        }

        #endregion
        #region - Properties -

        /// <summary>
        /// Gets or Sets the Name for this Airport.
        /// </summary>
        public String Name { get; set; }

        /// <summary>
        /// Gets or Sets the Waypoint for this Object.
        /// </summary>
        public Waypoint Waypoint { get; set; }

        #endregion
        #region - Public -

        /// <summary>
        /// Gets the string representation for this Airport.
        /// </summary>
        /// <returns>The string representation for this Airport.</returns>
        public override string ToString()
        {
            return this.Name;
        }

        #endregion
    }
}
