﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using FlightPlan.Optimizer.Wfs.FeatureTypes;

namespace FlightPlan.Optimizer.Airports
{
    /// <summary>
    /// Represents a list of Airports.
    /// It's initialized with some data.
    /// </summary>
    public class ListOfAirports : List<Airport>
    {
        #region - Constructors -

        /// <summary>
        /// Creates a new instance of ListOfAirports and add to it some data.
        /// </summary>
        public ListOfAirports()
            :base()
        {
            InitAirports();
        }

        #endregion
        #region - Public -

        /// <summary>
        /// Gets the closest airport to all the specified features.
        /// </summary>
        /// <param name="features">A list of FeatureTypes.</param>
        /// <returns>The closest airport to all the specified features.</returns>
        public Airport GetClosestAirport(ListOfFeaturesTypes features)
        {
            Airport closest = null;
            double minDist = double.MaxValue;
            double dist;

            // Search the airport with minimun distance to any feature
            foreach (AbstractFeatureType f in features)
            {
                foreach (Airport a in this)
                {
                    dist = f.The_Geom_Waypoint.Distance(a.Waypoint);

                    if (dist < minDist)
                    {
                        minDist = dist;
                        closest = a;
                    }
                }
            }


            return closest; 
        }

        #endregion
        #region - Private -

        /// <summary>
        /// Adds some data to this list.
        /// </summary>
        private void InitAirports()
        {
            // Aeropuertos
            Add(new Airport("Aeropuerto de A Coruña (LCG)", 43.294934,-8.383191));
            Add(new Airport("Aeropuerto de Alicante (ALC)", 38.279856,-0.541624));
            Add(new Airport("Aeropuerto de Bilbao (BIO)", 43.306256,-2.924809));
            Add(new Airport("Aeropuerto de Barcelona (BCN)", 41.287755, 2.084833));
            Add(new Airport("Aeropuerto de Ibiza (IBZ)", 38.878673,1.387142));
            Add(new Airport("Aeropuerto de Lleida-Alguaire (ILD)", 41.720913,0.545325));
            Add(new Airport("Aeropuerto de Madrid-Barajas (MAD)", 40.473624,-3.536246));
            Add(new Airport("Aeropuerto de Mallorca (PMI)", 39.555264,2.761323));
            Add(new Airport("Aeropuerto de Murcia (ILD)", 37.782044, -0.802989));
            Add(new Airport("Aeropuerto de Oviedo (OVD)", 43.560584,-6.021967));
            Add(new Airport("Aeropuerto de San Sebastián (EAS)", 43.350507,-1.797252));
            Add(new Airport("Aeropuerto de Santander (SDR)", 43.423674,-3.807224));
            Add(new Airport("Aeropuerto de Santiago de Compostela(SCQ)", 42.88403,-8.411043));
            Add(new Airport("Aeropuerto de Sevilla (SEV)", 37.418061,-5.87436));
            Add(new Airport("Aeropuerto de Reus-Tarragona (REU)", 41.15196, 1.182919));
            Add(new Airport("Aeropuerto de Vigo (VGO)", 42.221439,-8.629621));
            Add(new Airport("Aeropuerto de Valencia (VLC)", 39.483723,-0.466833));
            
            
            // AVIALSA, alfabético por provincias
            Add(new Airport("Aeródromo de Muchamiel (Alicante)", 38.438714,-0.470669));
            Add(new Airport("Aeródromo de Sabadell (Barcelona)", 41.518555, 2.10916));
            Add(new Airport("Aeródromo de Igualada (Barcelona)", 41.580886, 1.654547));
            Add(new Airport("Aeródromo de Castellón (Castellón), ", 40.002323,0.026479));
            Add(new Airport("Base contra incendios de Vistabella (Castellón)", 40.334498, -0.305707));
            Add(new Airport("Aeródromo de Empuriabrava (Girona)", 42.256944,3.110976));
            Add(new Airport("Base contra incendios de Ayora (Valencia)", 38.944165,-0.762681));
            Add(new Airport("Base contra incendios de Siete Aguas (Valencia)", 39.490811,-0.905063));
        }

        #endregion
    }
}
