﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;

using Utils;
using FlightPlan.Optimizer.TSPAlgorithms;
using FlightPlan.Optimizer.Wfs;
using FlightPlan.Optimizer.Wfs.FeatureTypes;
using System.Windows.Forms;

namespace FlightPlan.Optimizer
{
    public class Program
    {
        /// <summary>
        /// Helper method for loading a flight plan.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public static FlightPlan LoadFP(string fileName)
        {
            // Load flight plan contents into xml_contents
            //Assembly asm = Assembly.GetExecutingAssembly();
            //Stream str = asm.GetManifestResourceStream(fileName);
            string xml_contents;// = new StreamReader(str).ReadToEnd();


            xml_contents = File.ReadAllText(fileName);

            // Parse flight plan
            XMLFPParser fp_parser = new XMLFPParser();
            fp_parser.SetContent(xml_contents);
            return fp_parser.Parse();
        }

        public static void TestPrevLegs(FlightPlan fp)
        {
            foreach (Stage s in fp.stages)
            {
                foreach (Leg l in s.legsCol)
                {
                    Leg p = l.Prev;
                    Leg n = l.Next;

                    String strP = p == null ? "[null]" : p.ID;
                    String strN = n == null ? "[null]" : n.ID;

                    Console.WriteLine("Prev test succesfull {0} -> {1} -> {2}", strP, l.id, strN);
                }
            }
        }


        private static void TestWfsFeaturesTypes()
        {
            //ListOfFires lf = new ListOfFires(FeatureType.Fires30Days, "c:/users/sanjiser/downloads/FiresAll.csv");
            //ListOfHotspots lh = new ListOfHotspots(FeatureType.Hotspots7Days, "c:/users/sanjiser/downloads/HotspotsAll(1).csv");

            //WfsDataDownloader dd = new WfsDataDownloader();
            //String path = dd.DownloadData(FeatureType.FiresAll);
            //ListOfFires llf = new ListOfFires(FeatureType.FiresAll, path);
            //Console.WriteLine("hecho: {0}", path);


            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            //Application.Run(new GUI.WfsFiresForm());
            //Application.Run(new GUI.WfsHotspotsForm());
            Application.Run(new GUI.WfsSelectForm());


            Console.ReadLine();

        }

        // E:\ISIS\FlightPlanMonitor\GUI Elements\FlightPlanManagmet.xaml.cs --> donde está el fp

        [STAThread()]
        public static void Main(string[] args)
        {
            TestWfsFeaturesTypes();
            return;

            // Read a flighPlan
            FlightPlan fp2 = LoadFP("E:/ISIS/FlightPlanManagerTests/Beechcraft-LEBL-BasicScan.xml");
            TestPrevLegs(fp2);


            // Creates the flightplan
            FlightPlan fp;

            Console.WriteLine("Creating Test FP...");
            fp = FPGenerator.GetTestFlightPlan();
            TestPrevLegs(fp);
            Console.WriteLine("done!");

            ////////// !!!!!!!!!!!!!!!!!!!!!!111
            Console.WriteLine("Creating Optimized FP...");
            fp = FPGenerator.GetOptimizedFlighPlan(WayPointsGenerator.GetPath1(), TSPAlgorithmType.TypeGA);
            TestPrevLegs(fp);
            Console.WriteLine("done!");

            // Writes it to a file
            String path = "E:/ISIS/FlightPlanManagerTests/demoFP.xml";
            //String path = "demoFP.xml";
            Console.WriteLine("Writing FP to {0}", path);
            XMLFPWriter.Save(fp, path);
            Console.WriteLine("FP writed!");

            // Try to reload the previous flight plan
            FlightPlan fp3 = LoadFP(path);

            Console.ReadLine();

            //Function2();
            //Console.ReadLine();
        }

        public static void Function2()
        {
            // Read a flighPlan
            string path = "E:/ISIS/FlightPlanManagerTests/FP_KML.xml";
            Console.WriteLine("Loading {0}...", path);
            FlightPlan fp = LoadFP(path);
            //TestPrevLegs(fp2);
            Console.WriteLine("Done", path);

            path = "E:/ISIS/FlightPlanManagerTests/demoFP.xml";
            Console.WriteLine("Writing FP to {0}", path);
            XMLFPWriter.Save(fp, path);
            Console.WriteLine("FP writed!");


            Console.WriteLine("Optimizing FP...");
            Waypoint[] points = fp.GetAllWayPoints();
            fp = FPGenerator.GetOptimizedFlighPlan(points, TSPAlgorithmType.TypeGA);
            TestPrevLegs(fp);
            Console.WriteLine("done!");

            // Writes it to a file
            path = "E:/ISIS/FlightPlanManagerTests/demoFPOpt.xml";
            //String path = "demoFP.xml";
            Console.WriteLine("Writing FP to {0}", path);
            XMLFPWriter.Save(fp, path);
            Console.WriteLine("FP writed!");
        }
    }
}
