﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FlightPlan.Optimizer.Wfs
{
    #region - Enums -

    /// <summary>
    /// Enums the distinct feature types that could be downloaded from the Wfs server in the GetFeature request.
    /// </summary>
    public enum FeatureType
    {
        Hotspots1Day,
        Hotspots7Days,
        HotspotsAll,
        Fires7Days,
        Fires30Days,
        FiresAll
    }

    public enum ResultType
    {
        Results,
        Hits
    }

    public enum OutpuFormat
    {
        text_xml_gml311,
        gml2,
        gml2_gzip,
        shape_zip,
        csv,
        gml3,
        gml32,
        json,
        text_xml_gml212,
        text_xml_32
    }

    #endregion
    #region - Utility Class -

    /// <summary>
    /// Utility class to work with the Enums at this Namespace.
    /// </summary>
    public static class Enums
    {
        public static String FeatureType2Str(FeatureType ft)
        {
            switch (ft)
            {
                case FeatureType.Hotspots1Day:  return "EFFIS:Hotspots1Day";
                case FeatureType.Hotspots7Days: return "EFFIS:Hotspots7Days";
                case FeatureType.HotspotsAll:   return "EFFIS:HotspotsAll";
                case FeatureType.Fires7Days:    return "EFFIS:Fires7Days";
                case FeatureType.Fires30Days:   return "EFFIS:Fires30Days";
                case FeatureType.FiresAll:      return "EFFIS:FiresAll";
                default: throw new ArgumentException("ft isn't valid");
            }
        }

        public static String ResultType2Str(ResultType rt)
        {
            switch (rt)
            {
                case ResultType.Results: return "results";
                case ResultType.Hits:    return "hits";
                default:                 throw new ArgumentException("rt isn't valid");
            }
        }

        public static String OutputFormat2Str(OutpuFormat of)
        {
            switch (of)
	        {
		        case OutpuFormat.text_xml_gml311: return "text/xml; subtype=gml/3.1.1";
                case OutpuFormat.gml2:            return "GML2";
                case OutpuFormat.gml2_gzip:       return "GML2-GZIP";
                case OutpuFormat.shape_zip:       return "SHAPE-ZIP";
                case OutpuFormat.csv:             return "csv";
                case OutpuFormat.gml3:            return "gml3";
                case OutpuFormat.gml32:           return "gml32";
                case OutpuFormat.json:            return "json";
                case OutpuFormat.text_xml_gml212: return "text/xml; subtype=gml/2.1.2";
                case OutpuFormat.text_xml_32:     return "text/xml; subtype=gml/3.2";
                default: throw new ArgumentException("of isn't valid");
	        }
        }
    }

    #endregion
}
