﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Net;

namespace FlightPlan.Optimizer.Wfs
{
    /// <summary>
    /// Provides functions to download the last data from the Wfs Web Server.
    /// </summary>
    public class WfsDataDownloader
    {
        #region - Constructors -

        /// <summary>
        /// Creates a new instance of WfsDataDownloader.
        /// </summary>
        public WfsDataDownloader()
        {
            String s = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            this.DirectoryPath = Path.Combine(s, "EFFIS_data");
        }

        /// <summary>
        /// Creates a new instance of WfsDataDownloader.
        /// </summary>
        /// <param name="directoryPath">The directory where to save the downloaded data.</param>
        public WfsDataDownloader(String directoryPath)
        {
            this.DirectoryPath = directoryPath;
        }

        #endregion
        #region - Properties -

        /// <summary>
        /// The directory where to save the downloaded data.
        /// </summary>
        public String DirectoryPath { get; set; }

        #endregion
        #region - Public -

        /// <summary>
        /// Downloads from the wfs server the data specified at the url.
        /// </summary>
        /// <param name="url">The factory with the url information.</param>
        /// <returns>The path to the file where the the data were downloaded.</returns>
        public String DownloadData(WfsUrlFactory url)
        {
            // Ensure directory exists
            if (!Directory.Exists(DirectoryPath))
                Directory.CreateDirectory(DirectoryPath);

            String type = url.FeatureType.ToString();
            String path = Utils.CreateUniqueFileName(DirectoryPath, type, ".csv");

            WebClient webClient = new WebClient();
            webClient.DownloadFile(url.GetUrl(), path);

            return path;
        }

        /// <summary>
        /// Downloads from the wfs server the data of the specified type.
        /// </summary>
        /// <param name="type">The type of data to be downloaded.</param>
        /// <returns>The path to the file where the the data were downloaded.</returns>
        public String DownloadData(FeatureType type)
        {
            WfsUrlFactory fac = new WfsUrlFactory();
            fac.FeatureType = type;
            return DownloadData(fac);
        }

        #endregion
    }
}
