﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FlightPlan.Optimizer.Wfs
{
    /// <summary>
    /// Utility class to create urls to perform queries over the Wfs server.
    /// </summary>
    public class WfsUrlFactory
    {
        #region - Attributes -

        private string request           = "GetFeature";
        private FeatureType featureType  = FeatureType.Hotspots1Day;
        private OutpuFormat outputFormat = OutpuFormat.csv;
        private ResultType  resultType   = ResultType.Results;
        private int maxFeatures = -1;

        #endregion
        #region - Constructor -

        public WfsUrlFactory()
        {
            BaseUrl = "http://geohub.jrc.ec.europa.eu:80/effis/wfs";
        }

        #endregion
        #region - Properties -

        public String BaseUrl { get; set; }

        public FeatureType FeatureType
        {
            get { return featureType; }
            set { featureType = value; }
        }

        #endregion
        #region - Public -

        public String GetUrl()
        {
            String ftStr = Enums.FeatureType2Str(featureType);
            String ofStr = Enums.OutputFormat2Str(outputFormat);
            String rtStr = Enums.ResultType2Str(resultType);

            String s =
                String.Format(
                    "{0}?request={1}&typeName={2}&outputFormat={3}&resultType={4}",
                    BaseUrl,
                    request,
                    ftStr,
                    ofStr,
                    rtStr);

            if (maxFeatures > 0)
                s += "&maxFeatures=" + maxFeatures.ToString();

            return s;
        }

        #endregion
    }
}
