// ScanPoint.cs created with MonoDevelop
// User: esantama at 11:28 AM 11/17/2008

using System;
using System.Xml;

namespace FlightPlan
{
	/// <summary>
	/// Implements an eight pattern which can be used to scan a specific point.
	/// </summary>
	public class ScanPoint: Leg
    {
        #region - Attributes & Properties -

        /// <summary>
		/// Azimuth angle which determines the orientation of the eight pattern. 
		/// </summary>
		public double course;
        public double Course
        {
            get { return course; }
            set { course = value; }
        }

		/// <summary>
		/// Distance between turn centers of the eight pattern.
		/// </summary>
		public double d1;
        public double D1
        {
            get { return d1; }
            set { d1 = value; }
        }

		/// <summary>
		/// Diameter of the turn segments.
		/// </summary>
		public double d2;
        public double D2
        {
            get { return d2; }
            set { d2 = value; }
        }
		
		/// <summary>
		/// Returns d2/2
		/// </summary>
		public double radius 
        {
			get { return d2/2.0; }
		}

        #endregion
        #region - Constructors -

        //---------------------------------------------------------------------
		/// <summary>
		/// Creates a ScanPoint object.
		/// </summary>
		/// <param name="stg"><see cref="Stage"/> the leg belongs to.</param>
		/// <param name="id">A <see cref="System.String"/> with the leg id.</param>
		/// <param name="destination">A <see cref="Waypoint"/> with the initial fix of the holding pattern.</param>
		/// <param name="nextid">A <see cref="System.String"/> with the id of the next leg.</param>
		/// <param name="previd">A <see cref="System.String"/> with the id of the previous leg.</param>
		/// <param name="emergency">A <see cref="FlightPlan"/> to be executed in case of emergency.</param>
		/// <param name="cour">A <see cref="System.Double"/> with the azimuth angle indicating orientation of the eight pattern.</param>
		/// <param name="d1">A <see cref="System.Double"/> with the distance between turn centers.</param>
		/// <param name="d2">A <see cref="System.Double"/> with the turn diameter.</param>
		//---------------------------------------------------------------------				
		public ScanPoint(Stage stg, string id, Waypoint destination, string nextid,		                       
		                 string previd, ContingencyList emergency, double cour,  
		                 double d1, double d2)
			: base (stg, id, destination, nextid, previd, emergency)
		{
			course = cour;
			this.d1 = d1;
			this.d2 = d2;
		}

        #endregion
        #region - Public Functions -

        /// <summary>
        /// Returns a deep copy of an scan point object.
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        public override Leg DeepCopy(Stage s)
        {
            Leg l = base.DeepCopy(s);
            ScanPoint sp = new ScanPoint(l.stage, l.id, l.destination, l.nextid,
                l.previd, l.emergency, this.course, this.d1, this.d2);
            return sp;
        }

        #endregion
        #region - Xml -

        /// <summary>
        /// Gets the XsiType for this Leg.
        /// </summary>
        protected override String XsiType
        {
            get { return "fp:ScanPointLeg"; }
        }

        /// <summary>
        /// Appends this Object to a xml LegsCollection representation.
        /// </summary>
        /// <param name="legsElement">The LegsCollection XmlElement representation where to append this Object.</param>
        public override XmlElement AppendXml(XmlElement legsElement)
        {
            XmlElement xmlLeg = base.AppendXml(legsElement);

            XmlHelper.AddElement(xmlLeg, "course", course.ToString());
            XmlHelper.AddElement(xmlLeg, "d1",     d1.ToString());
            XmlHelper.AddElement(xmlLeg, "d2",     d2.ToString());

            return xmlLeg;
        }

        #endregion						
	}
}
