﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace FlightPlan
{
    /// <summary>
    /// Utility class to write a FlightPlan object to a xml file.
    /// </summary>
    /// <author>Santiago Jiménez Serrano [sanjiser@gmail.com]</author>
    /// <date>08/07/2011</date>
    public static class XMLFPWriter
    {
        #region - Public -

        /// <summary>
        /// Saves the specified FlightPlan to the specified path.
        /// </summary>
        /// <param name="fp">FlightPlan to be saved.</param>
        /// <param name="path">The path to the file in which to save the workflow.</param>
        public static void Save(FlightPlan fp, String path)
        {
            WriteXmlDocument(GetXmlDocument(fp), path);
        }

        #endregion
        #region - Private -

        /// <summary>
        /// Write the specified XML document to the specified path.
        /// </summary>
        /// <param name="xmlDoc">The XmlDocument to write.</param>
        /// <param name="path">The path in which save the document.</param>
        private static void WriteXmlDocument(XmlDocument xmlDoc, String path)
        {
            // Test for erros
            if (xmlDoc == null)       throw new ArgumentNullException("xmlDoc");
            if (path == String.Empty) throw new ArgumentNullException("path");

            // xml writer settings
            XmlWriterSettings writerSettings = GetXmlWriterSettings();

            // xml writer
            using (XmlWriter xmlWriter = XmlWriter.Create(path, writerSettings))
            {
                xmlDoc.Save(xmlWriter);
                xmlWriter.Close();
            }

            // Debug Info
            Console.WriteLine("{0} saved succesfully", path);
        }

        /// <summary>
        /// Gets a new XmlDocument with the XmlElement representing the specified FlightPlan.
        /// </summary>
        /// <param name="fp">The FlightPlan.</param>
        /// <returns>The XmlDocument containing the FlightPlan representation.</returns>
        private static XmlDocument GetXmlDocument(FlightPlan fp)
        {
            // Create the XmlDocument
            XmlDocument xmlDoc = new XmlDocument();

            // Create the elements
            XmlComment comment = xmlDoc.CreateComment("Auto-generated FlightPlan by FlightPlanXML.");
            XmlElement xmlFP   = fp.GetXmlElement(xmlDoc);

            // Append the elements
            xmlDoc.AppendChild(comment);
            xmlDoc.AppendChild(xmlFP);

            // return the XmlDocument
            return xmlDoc;
        }

        /// <summary>
        /// Creates a new XmlWriterSettings common for all the Xml Writer operations.
        /// </summary>
        /// <returns>A new XmlWriterSettings.</returns>
        private static XmlWriterSettings GetXmlWriterSettings()
        {
            XmlWriterSettings writerSettings = new XmlWriterSettings();
            writerSettings.Indent = true;
            writerSettings.NewLineOnAttributes = false;
            writerSettings.IndentChars = "\t";
            return writerSettings;
        }

        #endregion
    }
}
