﻿using System;
using System.Xml;

namespace FlightPlan
{
    /// <summary>
    /// Static class containing utility methods to work with xml nodes.
    /// </summary>    
    /// <see cref="http://csharp-slackers.blogspot.com/2008/09/quickly-append-nodes-in-xml-document_18.html"/>
    /// <author>Santiago Jiménez Serrano [sanjiser@gmail.com]</author>
    public static class XmlHelper
    {
        /// <summary>
        /// Adds a new XmlElement to the parent node with the specified paramenters.
        /// </summary>
        /// <param name="parent">The parent node.</param>
        /// <param name="tagName">The tag name.</param>
        /// <param name="textContent">The text content in the node.</param>
        /// <returns>The created and appended child node.</returns>
        public static XmlElement AddElement(XmlElement parent, string tagName, string textContent)
        {
            XmlElement node = parent.OwnerDocument.CreateElement(tagName);
            
            if (textContent != null)
            {   
                node.InnerXml = textContent;
            }

            parent.AppendChild(node);
            return node;
        }

        /// <summary>
        /// Adds a new XmlComment to the parent node with the specified parameters.
        /// </summary>
        /// <param name="parent">The parent node.</param>
        /// <param name="textComment">The text content of the comment.</param>
        /// <returns>The created and appended child node.</returns>
        public static XmlComment AddComment(XmlElement parent, String textComment)
        {
            XmlComment node = parent.OwnerDocument.CreateComment(textComment);
            parent.AppendChild(node);
            return node;
        }

        /// <summary>
        /// Adds a new attribute to the specified xml node.
        /// </summary>
        /// <param name="node">The xml node where to add the attribute.</param>
        /// <param name="attributeName">The attribute name.</param>
        /// <param name="textContent">The attribute value.</param>
        /// <returns>The new XmlAttribute object created.</returns>
        public static XmlNode AddAttribute(XmlNode node, string attributeName, string textContent)
        {
            XmlAttribute attribute = node.OwnerDocument.CreateAttribute(attributeName);
            attribute.Value = textContent;
            node.Attributes.Append(attribute);
            return attribute;
        }
    }
}
