﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using FlightPlan;

namespace FlightPlanXMLTests
{
    /// <summary>
    /// Summary description for FPCloningTests
    /// </summary>
    [TestClass]
    public class FPCloningTests
    {
        
        public FPCloningTests()
        {            
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void DeepCopyFP()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            Assert.IsNotNull(fp);

            FlightPlan.FlightPlan newfp = fp.DeepCopy();

            // Check flight plan fields
            Assert.AreNotSame(fp.description, newfp.description);
            Assert.AreEqual(fp.description, newfp.description);
            Assert.AreNotSame(fp.Fixes, newfp.Fixes);
            Assert.AreNotSame(fp.id, newfp.id);
            Assert.AreEqual(fp.id, newfp.id);
            // Locale info not expected to change between a fp and one of its copies
            Assert.AreSame(fp.LocaleInfo, newfp.LocaleInfo);
            Assert.AreNotSame(fp.name, newfp.name);
            Assert.AreEqual(fp.name, newfp.name);
            Assert.AreNotSame(fp.stages, newfp.stages);
            CheckStages(fp.stages, newfp.stages);
        }

        public void CheckStages(StageList sl1, StageList sl2)
        {
            Assert.AreNotSame(sl1, sl2);
            Assert.AreEqual(sl1.Count, sl2.Count);

            foreach (Stage s1 in sl1)
            {
                Stage s2 = sl2.Find(x => x.ID == s1.ID);
                Assert.AreNotSame(s1, s2);
                Assert.AreNotSame(s1.description, s2.description);
                Assert.AreEqual(s1.description, s2.description);
                Assert.AreNotSame(s1.finalLegs, s2.finalLegs);
                foreach (Leg l1 in s1.finalLegs)
                {
                    Leg l2 = s2.finalLegs.Find(x => x.ID == l1.ID);
                    Assert.AreNotSame(l1, l2);
                }                
                Assert.AreNotSame(s1.id, s2.id);
                Assert.AreEqual(s1.id, s2.id);
                Assert.AreNotSame(s1.initialLegs, s2.initialLegs);
                foreach (Leg l1 in s1.initialLegs)
                {
                    Leg l2 = s2.initialLegs.Find(x => x.ID == l1.ID);
                    Assert.AreNotSame(l1, l2);
                }                
                Assert.AreNotSame(s1.manualOnly, s2.manualOnly);
                Assert.AreEqual(s1.manualOnly, s2.manualOnly);
                Assert.AreNotSame(s1.name, s2.name);
                Assert.AreEqual(s1.name, s2.name);
                Assert.AreNotSame(s1.type, s2.type);
                Assert.AreEqual(s1.type, s2.type);
                Assert.AreNotSame(s1.flightPlan, s2.flightPlan);
                CheckLegs(s1.legsCol, s2.legsCol);
            }
        }

        public void CheckLegs(LegsCollection lc1, LegsCollection lc2)
        {
            // Note that only fields of base leg type are checked.
            Assert.AreNotSame(lc1, lc2);
            Assert.AreEqual(lc1.Count, lc2.Count);

            foreach (Leg l1 in lc1)
            {
                Leg l2 = lc2[l1.id];
                Assert.AreNotSame(l1, l2);
                CheckWaypoints(l1.destination, l2.destination);
                Assert.AreNotSame(l1.id, l2.id);
                if (l1.next != null)
                {
                    Assert.AreNotSame(l1.next, l2.next);
                    Assert.AreEqual(l1.next.id, l2.next.id);
                }
                else
                    Assert.IsNull(l2.next);
                if (l1.parent != null)
                {
                    Assert.AreNotSame(l1.parent, l2.parent);
                    Assert.AreEqual(l1.parent.id, l2.parent.id);
                }
                else
                    Assert.IsNull(l2.parent);
                if (l1.prev != null)
                {
                    Assert.AreNotSame(l1.prev, l2.prev);
                    Assert.AreEqual(l1.prev.id, l2.prev.id);
                }
                else
                    Assert.IsNull(l2.prev);
                Assert.AreNotSame(l1.stage, l2.stage);
                Assert.AreEqual(l1.stage.id, l2.stage.id);
            }
        }

        public void CheckWaypoints(Waypoint d1, Waypoint d2)
        {
            if (d1 == null)
            {
                Assert.IsNull(d2);
                return;
            }

            Assert.AreNotSame(d1, d2);
            if (d1.altitude != null)
            {
                Assert.AreNotSame(d1.altitude, d2.altitude);
                Assert.AreEqual(d1.altitude, d2.altitude);
            }
            else
                Assert.IsNull(d2.altitude);
            if (d1.fix != null)
            {
                Assert.AreNotSame(d1.fix, d2.fix);
                Assert.AreEqual(d1.fix.Id, d2.fix.Id);
            }
            else
                Assert.IsNull(d2.fix);
            Assert.AreNotSame(d1.flyOver, d2.flyOver);
            Assert.AreEqual(d1.flyOver, d2.flyOver);
            Assert.AreNotSame(d1.latitude, d2.latitude);
            Assert.IsTrue(Math.Abs(d1.latitude.AsDouble() - d2.latitude.AsDouble()) < 0.00001);
            Assert.AreNotSame(d1.leg, d2.leg);
            Assert.AreEqual(d1.leg.id, d2.leg.id);
            Assert.AreNotSame(d1.longitude, d2.longitude);
            Assert.IsTrue(Math.Abs(d1.longitude.AsDouble() - d2.longitude.AsDouble()) < 0.00001);
            if (d1.speed != null)
            {
                Assert.AreNotSame(d1.speed, d2.speed);
                Assert.AreEqual(d1.speed, d2.speed);
            }
            else
                Assert.IsNull(d2.speed);
            Assert.AreNotSame(d1.vas_id, d2.vas_id);
            Assert.AreEqual(d1.vas_id, d2.vas_id);
        }

        [TestMethod]
        public void DeepCopyFixes()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.VOR-HUE-b1900d.xml");
            Assert.IsNotNull(fp);

            FixesCollection fixes = fp.Fixes.DeepCopy();

            Assert.AreNotSame(fixes, fp.Fixes);
            Assert.AreEqual(fixes.Count, fp.Fixes.Count);

            foreach (Fix f in fixes)
            {
                RwyFix f1 = f as RwyFix;
                RwyFix f2 = fp.Fixes[f1.Id] as RwyFix;
                // Base fix type
                Assert.AreNotSame(f1.Id, f2.Id);
                Assert.AreEqual(f1.Id, f2.Id);
                Assert.AreNotSame(f1.Lat, f2.Lat);
                Assert.IsTrue(Math.Abs(f1.Lat.AsDouble() - f2.Lat.AsDouble()) < 0.00001);
                Assert.AreNotSame(f1.Lon, f2.Lon);
                Assert.IsTrue(Math.Abs(f1.Lon.AsDouble() - f2.Lon.AsDouble()) < 0.00001);
                Assert.AreNotSame(f1.Name, f2.Name);
                Assert.AreEqual(f1.Name, f2.Name);
                Assert.AreNotSame(f1.Description, f2.Description);
                Assert.AreEqual(f1.Description, f2.Description);
                // Rwy fix type
                Assert.AreNotSame(f1.Altitude, f2.Altitude);
                Assert.AreEqual(f1.Altitude, f2.Altitude);
                Assert.AreNotSame(f1.Heading, f2.Heading);
                Assert.AreEqual(f1.Heading, f2.Heading);
                Assert.AreNotSame(f1.Length, f2.Length);
                Assert.AreEqual(f1.Length, f2.Length);
            }
        }

    }
}
