﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using System.Reflection;
using System.IO;

using FlightPlan;

namespace FlightPlanXMLTests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class NextPrevTests
    {
        public NextPrevTests()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        /// <summary>
        /// Test next leg of all legs is accessible.
        /// </summary>
        /// <param name="fp"></param>
        private void NextAlwaysAccessible(FlightPlan.FlightPlan fp)
        {
            Stage s;
            for (int i = 0; i < fp.stages.Count - 1; i++)
            {
                s = fp.stages[i];
                foreach (Leg l in s.legsCol)
                    if (!(l is IntersectionLeg))
                        Assert.IsNotNull(l.Next, "{0} failed!", l.ID);
            }

            s = fp.stages[fp.stages.Count - 1]; // last stage
            foreach (Leg l in s.legsCol)
                if (!(l is IntersectionLeg))
                    Assert.IsTrue(l.Next != null || 
                        s.GetIndexOfFinal(l) >= 0, "{0} failed!", l.ID);
        }

        /// <summary>
        /// Test prev leg of all legs is accessible.
        /// </summary>
        /// <param name="fp"></param>
        private void PrevAlwaysAccessible(FlightPlan.FlightPlan fp)
        {
            Stage s = fp.stages[0]; // first stage
            foreach (Leg l in s.legsCol)
                if (!(l is IntersectionLeg))
                    Assert.IsTrue(l.Prev != null || s.GetIndexOfInitial(l) >= 0);

            for (int i = 1; i < fp.stages.Count; i++)
            {
                s = fp.stages[i];
                foreach (Leg l in s.legsCol)
                    if (!(l is IntersectionLeg))
                        Assert.IsNotNull(l.Prev, "{0} failed!", l.ID);
            }
        }
        
        /// <summary>
        /// Check that the next leg is always accessible.
        /// </summary>
        [TestMethod]
        public void NextLegAlwaysAccessibleFP1()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.HotSpotIgualada_BLegs.xml");
            Assert.IsNotNull(fp);

            NextAlwaysAccessible(fp);
        }

        /// <summary>
        /// Check that the previous leg is always accessible.
        /// </summary>
        [TestMethod]
        public void PrevLegAlwaysAccessibleFP1()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.HotSpotIgualada_BLegs.xml");
            Assert.IsNotNull(fp);
            
            PrevAlwaysAccessible(fp);
        }
        
        /// <summary>
        /// Check that the next leg is always accessible.
        /// </summary>
        [TestMethod]
        public void NextLegAlwaysAccessibleFP2()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.VOR-HUE-b1900d.xml");
            Assert.IsNotNull(fp);

            NextAlwaysAccessible(fp);
        }

        /// <summary>
        /// Check that the previous leg is always accessible.
        /// </summary>
        [TestMethod]
        public void PrevLegAlwaysAccessibleFP2()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.VOR-HUE-b1900d.xml");
            Assert.IsNotNull(fp);

            PrevAlwaysAccessible(fp);
        }

    }
}
