﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Reflection;
using System.IO;
using FlightPlan;

namespace FlightPlanXMLTests
{
    /// <summary>
    /// Summary description for UpdateChangeTests
    /// </summary>
    [TestClass]
    public class UpdateChangeTests
    {
        public UpdateChangeTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo en descripción y nombre de un FlightPlan
        /// </summary>

        [TestMethod]
        public void UpdateDescrptionandNameFlightPlan()
        {
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            Assert.AreNotEqual(fp.description, "Descripción nueva");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.Description-FlghtPlan.xml");
            FlightPlan.Updater updater = new Updater(update, fp);
            updater.Update();
            Assert.AreEqual(fp.name, "FPBasicoModificado");
            Assert.AreEqual(fp.description, "Descripción nueva");
        }
        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo en descripción y nombre de un stage
        /// </summary>
        [TestMethod]
        public void UpdateDescrptionandNameStage()
        {
            Stage stage;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.Description-Stage.xml");
            FlightPlan.Updater updater = new Updater(update, fp);
            updater.Update();
            try
            {
                stage = fp.stages.Find( x => x.name == "newName");
                Assert.AreEqual(stage.description, "new Description");
            }
            catch
            {
                Assert.Fail("Error al cambiar el nombre");
            }            
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo el campo dest de un leg
        /// </summary>
        [TestMethod]
        public void UpdateDestinationBasicLeg()
        {
            Stage stage;
            Leg leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.Destination-BasicLeg.xml");
            FlightPlan.Updater updater = new Updater(update, fp);
            updater.Update();
            stage = fp.stages.Find(x => x.id == "depart");
            leg = stage.legsCol.First(x => x.id == "L1");

            Assert.AreEqual(leg.destination.latitude.AsDouble(), 41.509626);
            Assert.AreEqual(leg.destination.longitude.AsDouble(), 2.1);
            Assert.AreEqual(leg.destination.altitude, 2000);
            Assert.AreEqual(leg.destination.speed, 200);
            Assert.AreEqual(leg.destination.flyOver, false);
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo el campo nest de un intersection
        /// </summary>
        [TestMethod]
        public void UpdateIntersectionLeg()
        {
            Stage stage;
            Leg leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.IntersectionLeg.xml");
            FlightPlan.Updater updater = new Updater(update, fp);

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "intersection");
            //Assert.AreEqual(leg.Next.id, "hold");
            
            updater.Update();

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "intersection");
            Assert.AreEqual(leg.Next.id, "radio");
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo a un RFLeg
        /// </summary>
        [TestMethod]
        public void UpdateRFLeg()
        {
            Stage stage;
            RFLeg leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.RFLeg.xml");
            FlightPlan.Updater updater = new Updater(update, fp);            

            updater.Update();

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "radio") as RFLeg;
            Assert.AreEqual(leg.destination.latitude.AsDouble(), 41.509626);
            Assert.AreEqual(leg.destination.longitude.AsDouble(), 2.1);
            Assert.AreEqual(leg.destination.altitude, 2000);
            Assert.AreEqual(leg.destination.speed, 200);

            Assert.AreEqual(Math.Round(leg.clatitude.AsDouble(),3), 41.455);
            Assert.AreEqual(Math.Round(leg.clongitude.AsDouble(), 3), 2.085);
            Assert.AreEqual(leg.direction ,Leg.TurnDir.Left);
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo a un HFLeg
        /// </summary>
        [TestMethod]
        public void UpdateHFLeg()
        {
            Stage stage;
            HFLeg leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.HFLeg.xml");
            FlightPlan.Updater updater = new Updater(update, fp);

            updater.Update();

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "hold") as HFLeg;

            Assert.AreEqual(Math.Round(leg.destination.latitude.AsDouble(), 3), 41.455);
            Assert.AreEqual(Math.Round(leg.destination.longitude.AsDouble(), 3), 2.175);
            Assert.AreEqual(leg.destination.altitude, 200);
            Assert.AreEqual(leg.destination.speed, 20);

            Assert.AreEqual(leg.direction, Leg.TurnDir.Right);
            Assert.AreEqual(leg.course, 220);
            Assert.AreEqual(leg.d1, 200);
            Assert.AreEqual(leg.d2, 250);
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo a un BasicScan
        /// </summary>
        [TestMethod]
        public void UpdateBasicScan()
        {
            Stage stage;
            BasicScan leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.BasicScan.xml");
            FlightPlan.Updater updater = new Updater(update, fp);

            updater.Update();

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "basicscan") as BasicScan;

            Assert.AreEqual(Math.Round(leg.destination.latitude.AsDouble(), 3), 41.455);
            Assert.AreEqual(Math.Round(leg.destination.longitude.AsDouble(), 3), 2.175);
            Assert.AreEqual(leg.destination.altitude, 200);
            Assert.AreEqual(leg.destination.speed, 20);

            Assert.AreEqual(leg.angle, 200);
            Assert.AreEqual(leg.dim1, 900);
            Assert.AreEqual(leg.dim2, 700);
            Assert.AreEqual(leg.separation, 20);
        }

        /// <summary>
        /// Comprueba el funcionamiento del update al aplicarlo a un IterativeLeg
        /// </summary>
        [TestMethod]
        public void UpdateIterativeLeg()
        {
            Stage stage;
            IterativeLeg leg;
            FlightPlan.FlightPlan fp = ResourceReader.LoadFP("FlightPlanXMLTests.FlightPlan-Basico.xml");
            string update = ResourceReader.LoadUpdate("FlightPlanXMLTests.UpdateXML.IterativeLeg.xml");
            FlightPlan.Updater updater = new Updater(update, fp);

            updater.Update();

            stage = fp.stages.Find(x => x.id == "traffic");
            leg = stage.legsCol.First(x => x.id == "iterative") as IterativeLeg;

            Assert.AreEqual(leg.first.id,"radio");
            Assert.AreEqual(leg.last.id, "L4");
            Assert.AreEqual(leg.upperBound,5);
        }
    }
}
