﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tsp.BB
{
    public class State2 : State
    {
        #region - Attributes -

        /// <summary>
        /// Valor de la parte conocida del camino, para el estado actual.
        /// </summary>
        public int pathValue = 0;

        #endregion
        #region - Constructors -

        /// <summary>
        /// Initial constructor for state2.
        /// </summary>
        public State2()
            :base() // iniciamos los otros atributos a 1
        {
            this.value = 0;

            int initialId = this.path[0];
            int max = int.MinValue;       // search the maximun in the minimun length paths
            for (int i = 1; i <= N; i++)
            {
                if (TheGraph.floydM[i, initialId] != int.MaxValue &&
                    !this.PathContainsNode(i) &&
                    TheGraph.floydM[i, initialId] > max)
                {
                    max = TheGraph.floydM[i, initialId];
                }
            }

            this.value += max*2; // hay que ir y volver
        }


        private State2(State2 parent, int idNewNode)
            :base(parent, idNewNode, -1)
        {
            this.pathValue = parent.pathValue + TheGraph.GetWeight(path[this.k - 2], path[this.k - 1]);
            this.value = this.pathValue;

            if (this.k == N) // solution ?
            {
                if (TheGraph.ExistLink(idNewNode, path[0]))
                    this.value += TheGraph.GetWeight(idNewNode, path[0]);
                else
                    this.value = int.MaxValue;
            }
            else
            {
                int initialId = this.path[0];
                int max = int.MinValue;       // search the maximun in the minimun length paths
                int maxIndex = -1;


                for (int i = 1; i <= N; i++)
                {
                    if (TheGraph.floydM[i, initialId] != int.MaxValue && 
                        !this.PathContainsNode(i) &&
                        TheGraph.floydM[i, initialId] > max)
                    {
                        max = TheGraph.floydM[i, initialId];
                        maxIndex = i;
                    }
                }

                if (max != int.MinValue)
                {
                    this.value += max;
                    this.value += TheGraph.floydM[idNewNode, maxIndex]; // hay que llegar a ese nodo
                }
                else
                    this.value = int.MaxValue; // doesn't exist path to the starting point (any node contains a link with it).
            }
        }

        #endregion
        #region - Public -
              
        /// <summary>
        /// Branchs this node.
        /// Also sets the State.BestValue and State.BestSolution variables if arrive to such situation in the branch call.
        /// And also do a pasive Prune, i.e; it doesn't add childs worst than the State.BestSolution state.
        /// </summary>
        /// <returns>The list of resulting branched nodes. null if it's not possible branch this node.</returns>
        public override List<State> Branch()
        {
            int currentNode = path[k - 1];
            Node n = TheGraph.GetNode(currentNode);
            Link l;

            List<State> list = null;
            int id2;

            for (n.ResetIterator(); !n.EndIterator(); )
            {
                l = n.NextLink();
                id2 = l.node2.id;

                if (!this.PathContainsNode(id2))
                {
                    State2 s3 = new State2(this, id2);

                    if (s3.k == N) // solution
                    {
                        if (s3.value < State.BestValue) // if we have found a best solution, saved it
                        {
                            BestSolution = s3;
                            BestValue = s3.value;
                        }
                    }

                    else if (s3.value < State.BestValue) // not a solution
                    {
                        if (list == null)
                            list = new List<State>();
                        list.Add(s3);
                    }
                    else
                    {
                        State.NbPruned++;
                    }
                }
                    
            }

            return list;
        }

        #endregion
        #region - Private -

        private int AuxFinal()
        {
            this.value = 0;
            for (int i = 0; i < k - 1; i++)
            {
                this.value += TheGraph.adyM[this.path[i], this.path[i + 1]]; // actual path cost
            }
            return this.value;
        }

        #endregion
    }
}
